/*
** info.c - displays information about an ADE-XRC series card.
**
** Utility for:
**
**   o ADE-XRC
**
** (C) Copyright 2008 Alpha Data
*/

#include <windows.h>
#include <stdio.h>
#include <string.h>

#include <adexrc.h>

#include "..\common\common.h"

static char bnames[][32] = {
    "ADE-XRC"
};


static const char*
spacename(
    int                 index,
    ADEXRC_BOARD_TYPE   boardtype)
{
    if (index == 0) {
        return "FPGA";
    } else {
        switch (boardtype) {
        case ADEXRC_BOARD_ADEXRC:
            if (index == 1) {
                return "control";
            } else {
                return "unknown!";
            }
        default:
            return "unknown!";
        }
    }
}

int
main(int argc, char * argv[])
{
    int                 ret = 0;
    ADMXRC2_HANDLE      card = NULL;
    ADEXRC_HANDLE       card2 = NULL;
    ADEXRC_STATUS       status;
    ADEXRC_VERSION_INFO versionInfo;
    ADEXRC_CARD_INFO    cardInfo;
    char                bname[256];
    DWORD               i;
    Option              options[2];
    Arguments           arguments;
    DWORD               readreqsize;
    DWORD               data;
    
    options[0].key = "card";
    options[0].type = Option_uint;
    options[0].def.uintVal = 0;
    options[0].meaning = "ID of card to use";
    options[1].key = "index";
    options[1].type = Option_uint;
    options[1].def.uintVal = 0;
    options[1].meaning = "index of card to use";
    arguments.nOption = 2;
    arguments.option = options;
    arguments.nParamType = 0;
    arguments.minNParam = 0;
    
    parse_command_line(
        "info",
        argc,
        argv,
        &arguments);

    if (options[0].specified && options[1].specified) {
        printf("Do not specify both /card and /index\n");
        ret = -1;
        goto done;
    }

    if (!options[1].specified) {
        ADMXRC2_CARDID cardID = options[0].value.uintVal;
        ADEXRC_CARDID card2ID;

        status = ADMXRC2_OpenCard(cardID, &card);
        if (status != ADMXRC2_SUCCESS) {
            printf("Failed to open PCI-X card with ID %ld: %s\n",
                (unsigned long) cardID, ADMXRC2_GetStatusString(status));
            ret = -1;
            goto done;
        }
    
	    /*
		 * Set the bridge ID for the PCIe card. We can either use this value
		 * to open a specific card, or 0 to open the first one we find.
	     */
		data = 0x12345678;
	    ADMXRC2_Write(card,ADMXRC2_IOWIDTH_32,0,0x10,&data,4);

		card2ID = 0x12345678;   /* Or 0 to open the first/only card */
        status = ADEXRC_OpenCard(&card2, card2ID);
        if (status != ADEXRC_SUCCESS) {
            printf("Failed to open PCIe card with ID 0x%lx: %s\n",
                (unsigned long) card2ID, ADEXRC_GetStatusString(status));
            ret = -1;
            goto done;
        }
    } else {
        unsigned int index = options[1].value.uintVal;

        status = ADMXRC2_OpenCardByIndex(index, &card);
        if (status != ADMXRC2_SUCCESS) {
            printf("Failed to open PCI-X card with index %ld: %s\n",
                (unsigned long) index, ADMXRC2_GetStatusString(status));
            ret = -1;
            goto done;
        }

        status = ADEXRC_OpenCard(&card2, index);
        if (status != ADEXRC_SUCCESS) {
            printf("Failed to open PCIe card with index %ld: %s\n",
                (unsigned long) index, ADEXRC_GetStatusString(status));
            ret = -1;
            goto done;
        }
    }

    status = ADEXRC_GetVersionInfo(card2, &versionInfo);
    if (status != ADEXRC_SUCCESS) {
        printf("Failed to get version info: %s\n",
            ADEXRC_GetStatusString(status));
        ret = -1;
        goto done;
    }

    status = ADEXRC_GetCardInfo(card2, &cardInfo);
    if (status != ADEXRC_SUCCESS) {
        printf("Failed to get card info: %s\n",
            ADEXRC_GetStatusString(status));
        ret = -1;
        goto done;
    }

    if (cardInfo.BoardType < 0 || cardInfo.BoardType >= ADEXRC_BOARD_UNKNOWN) {
        sprintf(bname, "unknown (0x%lx)", (unsigned long) cardInfo.BoardType);
    } else {
        strncpy(bname, bnames[cardInfo.BoardType], 255);
    }

    printf("API ver                      %ld.%ld\n",
        (unsigned long) versionInfo.APIMajor,
        (unsigned long) versionInfo.APIMinor);
    printf("Driver ver                   %ld.%ld\n",
        (unsigned long) versionInfo.DriverMajor,
        (unsigned long) versionInfo.DriverMinor);
    printf("\n");

    printf("Board Type                   %s\n", bname);
    printf("Card ID                      %ld (0x%4.4lX)\n",
        (unsigned long) cardInfo.CardID,
        (unsigned long) cardInfo.CardID);
    printf("Bridge ID                    %ld (0x%4.4lX)\n",
        (unsigned long) cardInfo.BridgeID,
        (unsigned long) cardInfo.BridgeID);
    printf("Bridge Rev                   %d.%d\n",
        cardInfo.BridgeRevision >> 16, cardInfo.BridgeRevision & 0xFFFF);
    printf("\n");

    printf("Number of DMA channels       %ld\n",
        (unsigned long) cardInfo.NumDMAChan);
    printf("Memory Size                  %ld\n",
        (unsigned long) cardInfo.MemorySize);
    printf("Write Packet Size            %ld\n",
        (unsigned long) cardInfo.WrPktSize);
    printf("Read Packet Size             %ld\n",
        (unsigned long) cardInfo.RdPktSize);
    printf("Number of Packets in Flight  %ld\n",
        (unsigned long) cardInfo.PktsInFlight);
    printf("PCIe Configured Max Payload  X%ld\n",
        (unsigned long) (cardInfo.PCIEStatus & 0x00000007));
    printf("PCIe Programmed Max Payload  X%ld\n",
        (unsigned long) ((cardInfo.PCIEStatus & 0x00000700) >> 8));
    readreqsize = 0;
    i = (unsigned long) ((cardInfo.PCIEStatus & 0x00070000) >> 16);
    switch(i) {
    case 0:
        readreqsize = 128;    /* 128 bytes */
        break;
    case 1:
        readreqsize = 256;    /* 256 bytes */
        break;
    case 2:
        readreqsize = 512;    /* 512 bytes */
        break;
    case 3:
        readreqsize = 1024;   /* 1024 bytes */
        break;
    case 4:
        readreqsize = 2048;   /* 2048 bytes */
        break;
    case 5:
        readreqsize = 4096;   /* 4096 bytes */
        break;
    }
    printf("PCIe Configured Max Read Request Size %ld (%ld bytes)\n", 
           i, readreqsize );
    printf("\n");
    
    /*
     * Read the registers individually
     */
    for( i = 0; i < 0x120; i+=4 ) {
        if( i > 0x4C && i < 0x100 ) {
            /* Unused registers, don't bother reading them */
            continue;
        }
        status = ADEXRC_ReadReg(card2, i, &data );
        if (status != ADEXRC_SUCCESS) {
            printf("Failed to get read register: %s\n",
                   ADEXRC_GetStatusString(status));
            ret = -1;
            goto done;
        }
        printf("PCIe Register 0x%lX = 0x%lX)\n", i, data ); 
    }
    printf("\n");
    
    for( i = 0; i < 0x20; i+=4 ) {
        status = ADEXRC_ReadReg(card2, 0x800+i, &data );
        if (status != ADEXRC_SUCCESS) {
            printf("Failed to get read register: %s\n",
                   ADEXRC_GetStatusString(status));
            ret = -1;
            goto done;
        }
        printf("PCIe Register 0x%lX = 0x%lX)\n", i, data ); 
    }
    printf("\n");

    /*
     * Write a couple of register, then read them back
     */
    printf("Writing 0x20 to PCIe Register 0x0C\n" ); 
    data = 0x20;
    status = ADEXRC_WriteReg(card2, 0x0C, data );
    if (status != ADEXRC_SUCCESS) {
        printf("Failed to write register: %s\n",
               ADEXRC_GetStatusString(status));
        ret = -1;
        goto done;
    }
    status = ADEXRC_ReadReg(card2, 0x0C, &data );
    if (status != ADEXRC_SUCCESS) {
        printf("Failed to get read register: %s\n",
               ADEXRC_GetStatusString(status));
        ret = -1;
        goto done;
    }
    printf("PCIe Register 0x%lX = 0x%lX)\n", 0x0C, data ); 
    printf("Writing 0x20 to PCIe Register 0x20\n" ); 
    data = 0x20;
    status = ADEXRC_WriteReg(card2, 0x20, data );
    if (status != ADEXRC_SUCCESS) {
        printf("Failed to write register: %s\n",
               ADEXRC_GetStatusString(status));
        ret = -1;
        goto done;
    }
    status = ADEXRC_ReadReg(card2, 0x20, &data );
    if (status != ADEXRC_SUCCESS) {
        printf("Failed to get read register: %s\n",
               ADEXRC_GetStatusString(status));
        ret = -1;
        goto done;
    }
    printf("PCIe Register 0x%lX = 0x%lX)\n", 0x20, data ); 
    printf("\n");
    
done:
    if (card != NULL) {
        ADMXRC2_CloseCard(card);
    }
    if (card2 != NULL) {
        ADEXRC_CloseCard(card2);
    }

    return ret;
}
