/*
** simple.c - Trivial example showing how to implement read/write registers
**            in the FPGA.
**
** Example for:
**
**   o ADE-XRC
**
** (C) Copyright 2008 Alpha Data
*/

#include <windows.h>
#include <stdio.h>

#include <adexrc.h>

#include "../common/common.h"

int main(int argc, char *argv[])
{
    int                 ret = 0;
    ADEXRC_STATUS      status;
    ADEXRC_HANDLE      card = NULL;
    ADEXRC_CARD_INFO   cardInfo;
    unsigned long       dataOut, dataIn;
    Option              options[2];
    Arguments           arguments;

    /* Set up expected options */
    options[0].key = "card";
    options[0].type = Option_uint;
    options[0].def.uintVal = 0;
    options[0].meaning = "ID of card to use";
    options[1].key = "index";
    options[1].type = Option_uint;
    options[1].def.uintVal = 0;
    options[1].meaning = "index of card to use";
    arguments.nOption = 2;
    arguments.option = options;
    arguments.nParamType = 0;
    arguments.minNParam = 0;

    /* Parse command line */
    parse_command_line(
        "simple",
        argc,
        argv,
        &arguments);

    if (options[0].specified && options[1].specified) {
        printf("Do not specify both /card and /index\n");
        ret = -1;
        goto done;
    }

    if (!options[1].specified) {
        ADEXRC_CARDID cardID = options[0].value.uintVal;

        status = ADEXRC_OpenCard(&card, cardID);
        if (status != ADEXRC_SUCCESS) {
            printf("Failed to open card with ID %ld: %s\n",
                (unsigned long) cardID, ADEXRC_GetStatusString(status));
            ret = -1;
            goto done;
        }
    } else {
        unsigned int index = options[1].value.uintVal;

        status = ADEXRC_OpenCard(&card, index);
        if (status != ADEXRC_SUCCESS) {
            printf("Failed to open card with index %ld: %s\n",
                (unsigned long) index, ADEXRC_GetStatusString(status));
            ret = -1;
            goto done;
        }
    }

    status = ADEXRC_GetCardInfo(card, &cardInfo);
    if (status != ADEXRC_SUCCESS) {
        printf("Failed to get card info: %s\n",
            ADEXRC_GetStatusString(status));
        ret = -1;
        goto done;
    }

    switch (cardInfo.BoardType) {
    case ADEXRC_BOARD_ADEXRC:
        break;

    default:
        printf("*** This example must be run on one of the following cards:\n\n");
        printf("\tADE-XRC\n");
        ret = -1;
        goto done;
    }

    printf("====================\n" );
    //printf("Enter values for I/O\n" );
    //printf(" (use 55AA to exit)\n");
    printf("====================\n" );

    while (1) {
        scanf("%lx", &dataOut);
        //status = ADEXRC_WriteReg(card, 0x100, dataOut );
        /*if (status != ADEXRC_SUCCESS) {
            printf("Failed to write register: %s\n",
                   ADEXRC_GetStatusString(status));
            ret = -1;
            goto done;
        }*/
		if (dataOut == 0x55AAU) {
            break;
		printf(".\n");
        status = ADEXRC_ReadReg(card, dataOut, &dataIn );
        if (status != ADEXRC_SUCCESS) {
            printf("Failed to get read register: %s\n",
                   ADEXRC_GetStatusString(status));
            ret = -1;
            goto done;
        }
		printf("Read Addr %8.8lx: %8.8lx\n",
               (unsigned long) dataOut,
               (unsigned long) dataIn);
        
        }
    }

done:
    if (card != NULL) {
        ADEXRC_CloseCard(card);
    }
    
    return ret;
}
