/*
 * adexrc.h -  ADE-XRC SDK API header file (ADEXRC interface)
 *
 * (C) Copyright 2008 Alpha Data Parallel Systems Ltd.
 */

#ifndef _ADATA_ADEXRC_H
#define _ADATA_ADEXRC_H

#include <windows.h>

#if defined _MSC_VER
    /* MSVC++ */
#   if defined ADEXRC_DLL
        /* Compiling API library into a DLL */
#       define ADEXRC_DLLFUNC __declspec(dllexport)
#   else
        /* Importing API library from DLL */
#       define ADEXRC_DLLFUNC __declspec(dllimport)
#   endif
#else
    /* Borland or other C++ */
#   define ADEXRC_DLLFUNC
#endif

#include <adexrc\status.h>
#include <adexrc\struct.h>

/*
** Function prototypes
*/
#ifdef __cplusplus
extern "C"{
#endif

ADEXRC_DLLFUNC const char* __cdecl
ADEXRC_GetStatusString(         ADEXRC_STATUS Status);

ADEXRC_DLLFUNC ADEXRC_STATUS __cdecl
ADEXRC_StatusToString(          ADEXRC_STATUS status,
                                char*         string,
                                unsigned long length);

ADEXRC_DLLFUNC ADEXRC_STATUS __cdecl
ADEXRC_InstallErrorHandler(     ADEXRC_ERROR_HANDLER handler);

ADEXRC_DLLFUNC ADEXRC_STATUS __cdecl
ADEXRC_OpenCard(                ADEXRC_HANDLE* card,
                                UINT32         id);

ADEXRC_DLLFUNC ADEXRC_STATUS __cdecl
ADEXRC_CloseCard(               ADEXRC_HANDLE  card);

ADEXRC_DLLFUNC ADEXRC_STATUS __cdecl
ADEXRC_GetCardInfo(             ADEXRC_HANDLE     card,
                                ADEXRC_CARD_INFO* info);

ADEXRC_DLLFUNC ADEXRC_STATUS __cdecl
ADEXRC_SetupDMA(                ADEXRC_HANDLE   card,
                                const void*     buffer,
                                unsigned long   size,
                                DWORD           flags,  /* Currently Must be 0 */
                                ADEXRC_DMADESC* dmadesc);

ADEXRC_DLLFUNC ADEXRC_STATUS __cdecl
ADEXRC_UnsetupDMA(              ADEXRC_HANDLE   card,
                                ADEXRC_DMADESC  dmadesc);

ADEXRC_DLLFUNC ADEXRC_STATUS __cdecl
ADEXRC_DoDMA(                   ADEXRC_HANDLE   card,
                                ADEXRC_DMADESC  dmadesc,
                                ADEXRC_DMADIR   direction,
                                unsigned long   offset, /* Ignored, assumed zero */
                                unsigned long   length, /* Ignored, assumed buffer size */
                                DWORD           flags,  /* Ignored */
                                unsigned long*  timeout,/* Ignored by Win32, Used by Linux */
                                HANDLE          event); /* Used by Win32, Ignored by Linux */

ADEXRC_DLLFUNC ADEXRC_STATUS __cdecl
ADEXRC_WriteReg(				ADEXRC_HANDLE	card,
								unsigned int	address,
								unsigned int	value);

ADEXRC_DLLFUNC ADEXRC_STATUS __cdecl
ADEXRC_ReadReg(					ADEXRC_HANDLE	card,
								unsigned int	address,
								unsigned int*	value);

ADEXRC_DLLFUNC ADEXRC_STATUS __cdecl
ADEXRC_GetVersionInfo(          ADEXRC_HANDLE        card,
                                ADEXRC_VERSION_INFO* info);

#ifdef __cplusplus
} 
#endif

#endif /* ADEXRC_H */

