/*
 * status.h - status codes returned by ADE-XRC API calls
 *
 * (C) Copyright 2008 Alpha Data Parallel Systems Ltd.
 */

#ifndef _ADATA_ADEXRC_STATUS_H
#define _ADATA_ADEXRC_STATUS_H

/*
 * Base value for error status codes
 */
#define ADEXRC_STATUS_ERROR_CODE  (0x2000)

/*
 * Status and error codes
 */
typedef enum _ADEXRC_STATUS
{
	ADEXRC_SUCCESS             = 0,

	/* An error in the API logic occurred */
	ADEXRC_INTERNAL_ERROR		= ADEXRC_STATUS_ERROR_CODE,

	/* Couldn't allocate memory required to complete operation */
	ADEXRC_NO_MEMORY,

	/* Failed to open the card with specified CardID */
	ADEXRC_CARD_NOT_FOUND,

	/* The handle to the card passed was invalid */
	ADEXRC_INVALID_HANDLE,

	/* The operation was not completed within the timeout period */
	ADEXRC_TIMEOUT,

	/* Card could not be opened because it was already open */
	ADEXRC_CARD_BUSY,

	/* An invalid parameter was supplied to the call */
	ADEXRC_INVALID_PARAMETER,

	/* The card was closed before the operation was completed */
	ADEXRC_CLOSED,

	/* A hardware error occurred on the card */
	ADEXRC_CARD_ERROR,

	/* An operation was requested which is not supported or implemented */
	ADEXRC_NOT_SUPPORTED,

	/* The requested device or resource was in use */
	ADEXRC_DEVICE_BUSY,

	/* The DMA descriptor passed was invalid */
	ADEXRC_INVALID_DMADESC,

	/* No free DMA descriptors left */
	ADEXRC_NO_DMADESC,

	/* The operation failed */
	ADEXRC_FAILED,

	/* The operation is still in progress */
	ADEXRC_PENDING,

	/* The operation failed for reasons unknown */
	ADEXRC_UNKNOWN_ERROR,

	/* A null pointer was supplied in the call */
	ADEXRC_NULL_POINTER,

	/* The operation was cancelled because requesting thread terminated */
	ADEXRC_CANCELLED,

	/* The driver revision level is too low */
	ADEXRC_BAD_DRIVER

} ADEXRC_STATUS;

#define ADEXRC_IS_ERROR(Status) ((Status) >= ADEXRC_STATUS_ERROR_CODE)

#endif

