/*
 * struct.h - structures and types used in ADE-XRC API calls
 *
 * (C) Copyright 2008 Alpha Data Parallel Systems Ltd.
 */

#ifndef _ADATA_ADEXRC_STRUCT_H
#define _ADATA_ADEXRC_STRUCT_H

/*
 * A value to indicate which card
 */
typedef unsigned long ADEXRC_DEVICE_NUM;

/*
 * Handle to a card
 */
typedef HANDLE ADEXRC_HANDLE;
#define ADEXRC_HANDLE_INVALID_VALUE (NULL)

/*
 * A type that holds a Card ID
 */
typedef unsigned long ADEXRC_CARDID;

/*
 * Card enumeration function type
 */
typedef void (*ADEXRC_ENUM_FUNCTION)(unsigned int index, ADEXRC_HANDLE card);

/*
 * Error handler function type
 */
typedef void (*ADEXRC_ERROR_HANDLER)(const char* FnName, ADEXRC_STATUS Status);

/*
 * Handle to a DMA descriptor
 */
typedef unsigned long ADEXRC_DMADESC;

/*
 * DMA direction options
 */
typedef enum _ADEXRC_DMADIR {
	ADEXRC_PCITOLOCAL = 0,
	ADEXRC_LOCALTOPCI = 1,
	ADEXRC_DMADIR_FORCE32BITS = 0x7FFFFFFFU
} ADEXRC_DMADIR;

/*
 * Bus width value
 */
typedef enum _ADEXRC_IOWIDTH {
	ADEXRC_IOWIDTH_8  = 0,
	ADEXRC_IOWIDTH_16 = 1,
	ADEXRC_IOWIDTH_32 = 2,
	ADEXRC_IOWIDTH_64 = 3,
	ADEXRC_IOWIDTH_FORCE32BITS = 0x7FFFFFFFU
} ADEXRC_IOWIDTH;

/*
** ADEXRC_DoDMA() flags
*/
#define ADEXRC_DMAFLAG_CANQUEUE          (0x0 << 0)	/* default */
#define ADEXRC_DMAFLAG_DONOTQUEUE        (0x1 << 0)

/*
 * DMA channel options
 */
typedef enum _ADEXRC_DMA_CHANNEL
{
    ADEXRC_DMACHAN_0 = 0,
    ADEXRC_DMACHAN_1 = 1,
    ADEXRC_DMACHAN_ANY = 0xFFU
}
ADEXRC_DMA_CHANNEL;

/*
 * ADEXRC_Read() and ADEXRC_Write() flags
 */
#define ADEXRC_IOBYTE   (0)
#define ADEXRC_IOWORD   (1)
#define ADEXRC_IOLONG   (2)

#define ADEXRC_IOLOCAL    (0<<0)
#define ADEXRC_IOFIXED    (0<<1)
#define ADEXRC_IOADAPTER  (1<<1)

/*
 * ADE-XRC board types, used in ADEXRC_CARD_INFO2::BoardType
 */
typedef enum _ADEXRC_BOARD_TYPE {
	ADEXRC_BOARD_ADEXRC            = 0,	    /* ADE-XRC           */
	ADEXRC_BOARD_UNKNOWN		   = 1,
	ADEXRC_BOARD_FORCE32BITS       = 0x7FFFFFFFU
} ADEXRC_BOARD_TYPE;

/*
 * Card information structure
 */
typedef struct _ADEXRC_CARD_INFO {
	ADEXRC_CARDID		CardID;			/* ID of card								*/
	DWORD				BridgeID;		/* ID of bridge (set using ADM-XRC driver)	*/
	ADEXRC_BOARD_TYPE	BoardType;		/* type of card                             */
	unsigned long		NumDMAChan;		/* number of DMA channels                   */
	DWORD				BridgeRevision;	/* bridge rev, 0xaaaabbbb, a = maj, b = min */
	DWORD        		MemorySize;		/* Memory size on board 					*/
	DWORD        		PCIEStatus;		/* PCI-Express configuration status			*/
    WORD                WrPktSize;		/* Write Packet length                      */
    WORD                RdPktSize;		/* Read Packet length                       */
	BYTE				PktsInFlight;	/* Number of packets in flight              */
} ADEXRC_CARD_INFO;

/*
 * Structure returning version information
 */
typedef struct _ADEXRC_VERSION_INFO {
	BYTE DriverMinor;
	BYTE DriverMajor;
	BYTE APIMinor;
	BYTE APIMajor;
} ADEXRC_VERSION_INFO;

#endif
