/*
** admxrc.h -  ADM-XRC SDK API header file (ADMXRC interface)
**
** (C) Copyright 1999-2001 Alpha Data Parallel Systems Ltd.
*/

#ifndef _ADATA_ADMXRC_H
#define _ADATA_ADMXRC_H

#include <windows.h>

#if defined _MSC_VER
    /* MSVC++ */
#   if defined ADMXRC_DLL
        /* Compiling API library into a DLL */
#       define ADMXRC_DLLFUNC __declspec(dllexport)
#   else
        /* Importing API library from DLL */
#       define ADMXRC_DLLFUNC __declspec(dllimport)
#   endif
#else
	/* Borland or other C++ */
#   define ADMXRC_DLLFUNC
#endif

#include <admxrc\status.h>
#include <admxrc\struct.h>

/*
** Handle to a card
*/
typedef void *ADMXRC_HANDLE;

/*
** Error handler function type
*/
typedef void (*ADMXRC_HANDLER_FUNCTION)(const char* FnName, ADMXRC_STATUS Status);

/*
** FPGA Image file
*/
typedef void* ADMXRC_IMAGE;

/*
** Function prototypes
*/
#ifdef __cplusplus
extern "C"{
#endif

ADMXRC_DLLFUNC ADMXRC_STATUS __cdecl
ADMXRC_OpenCard(					ADMXRC_DEVICE_NUM		cardID,
									ADMXRC_HANDLE*			card);

ADMXRC_DLLFUNC ADMXRC_STATUS __cdecl
ADMXRC_CloseCard(					ADMXRC_HANDLE			card);

ADMXRC_DLLFUNC ADMXRC_STATUS __cdecl
ADMXRC_GetVersionInfo(				ADMXRC_HANDLE			card,
									ADMXRC_VERSION_INFO*	info);

ADMXRC_DLLFUNC ADMXRC_STATUS __cdecl
ADMXRC_GetCardInfo(					ADMXRC_HANDLE			card,
									ADMXRC_CARD_INFO*		info);

ADMXRC_DLLFUNC ADMXRC_STATUS __cdecl
ADMXRC_SetClockRate(				ADMXRC_HANDLE			card,
									ADMXRC_CLOCK			clock,
									double					rate);

ADMXRC_DLLFUNC ADMXRC_STATUS __cdecl
ADMXRC_ConfigureFromBuffer(			ADMXRC_HANDLE			card,
									void*					buffer,
									DWORD					length);

ADMXRC_DLLFUNC ADMXRC_STATUS __cdecl
ADMXRC_ConfigureFromBufferDMA(		ADMXRC_HANDLE			card,
									void*					buffer,
									DWORD					length,
									DWORD					dma_channel,// type is ADMXRC_DMA_CHANNEL
									PHANDLE					event);		// per-thread event handle

ADMXRC_DLLFUNC ADMXRC_STATUS __cdecl
ADMXRC_ConfigureFromFile(			ADMXRC_HANDLE			card,
									char*					fileName);

ADMXRC_DLLFUNC ADMXRC_STATUS __cdecl
ADMXRC_ConfigureFromFileDMA(		ADMXRC_HANDLE			card,
									char*					fileName,
									DWORD                   dma_channel,// type is ADMXRC_DMA_CHANNEL
									PHANDLE                 event);		// per-thread event handle

ADMXRC_DLLFUNC ADMXRC_STATUS __cdecl
ADMXRC_InstallErrorHandler(			ADMXRC_HANDLER_FUNCTION	handler);

ADMXRC_DLLFUNC ADMXRC_STATUS __cdecl
ADMXRC_ProgramBytes(				ADMXRC_HANDLE			card,
									unsigned long*			program,
									unsigned long			length);

ADMXRC_DLLFUNC ADMXRC_STATUS __cdecl
ADMXRC_WriteReg(					ADMXRC_HANDLE			card,
									unsigned char			index,
									unsigned char			value);

ADMXRC_DLLFUNC ADMXRC_STATUS __cdecl
ADMXRC_ReadReg(						ADMXRC_HANDLE			card,
									unsigned char			index,
									unsigned char*			value);

ADMXRC_DLLFUNC ADMXRC_STATUS __cdecl
ADMXRC_GetClockType(				ADMXRC_HANDLE			card,
									ADMXRC_CLOCK_TYPE*		value);

ADMXRC_DLLFUNC ADMXRC_STATUS __cdecl
ADMXRC_GetBaseAddress( 				ADMXRC_HANDLE			card,
									void**					pFPGA);

ADMXRC_DLLFUNC ADMXRC_STATUS __cdecl
ADMXRC_WaitForInterrupt(			ADMXRC_HANDLE			card,
									unsigned long*			timeout,	// timeout in microseconds
									unsigned long			flags,		// combination of ADMXRC_INTWAIT_XXX
									PHANDLE					event);		// per-thread event handle

ADMXRC_DLLFUNC ADMXRC_STATUS __cdecl
ADMXRC_RegisterInterruptEvent(		ADMXRC_HANDLE			card,
									HANDLE					event);

ADMXRC_DLLFUNC ADMXRC_STATUS __cdecl
ADMXRC_UnregisterInterruptEvent(	ADMXRC_HANDLE			card,
									HANDLE					event);

ADMXRC_DLLFUNC ADMXRC_STATUS __cdecl
ADMXRC_Read(						ADMXRC_HANDLE			card,
									unsigned long			width,
									unsigned long			flags,
									DWORD					local,
									void*					data,
									unsigned long			len);

ADMXRC_DLLFUNC ADMXRC_STATUS __cdecl
ADMXRC_Write(						ADMXRC_HANDLE			card,
									unsigned long			width,
									unsigned long			flags,
									DWORD					local,
									void*					data,
									unsigned long			len);

ADMXRC_DLLFUNC ADMXRC_STATUS __cdecl
ADMXRC_SetupDMA(					ADMXRC_HANDLE			card,
									void*					buffer,
									unsigned long			size,
									DWORD					flags,
									ADMXRC_DMADESC*			dmadesc);

ADMXRC_DLLFUNC ADMXRC_STATUS __cdecl
ADMXRC_UnsetupDMA(					ADMXRC_HANDLE			card,
									ADMXRC_DMADESC			dmadesc);

ADMXRC_DLLFUNC ADMXRC_STATUS __cdecl
ADMXRC_MapDirectMaster(				ADMXRC_HANDLE			card,
									ADMXRC_DMADESC			dmadesc,
									unsigned long           offset,
									unsigned long           length,
									ADMXRC_BUFFERMAP*		map);

ADMXRC_DLLFUNC ADMXRC_STATUS __cdecl
ADMXRC_SyncDirectMaster(			ADMXRC_HANDLE			card,
									ADMXRC_DMADESC			dmadesc,
									unsigned long			offset,
									unsigned long			length,
									DWORD                   syncmode);

ADMXRC_DLLFUNC ADMXRC_STATUS __cdecl
ADMXRC_DoDMA(						ADMXRC_HANDLE			card,
									ADMXRC_DMADESC			dmadesc,	// obtained from ADMXRC_SetupDMA()
									unsigned long			offset,		// offset into region mapped by dmadesc
									unsigned long			length,		// length of transfer
									DWORD					local,		// local bus address for DMA
									DWORD					direction,	// type is ADMXRC_DMA_DIRECTION
									DWORD					channel,	// type is ADMXRC_DMA_CHANNEL
									DWORD					mode,		// from ADMXRC_BuildDMAModeWord()
									DWORD					flags,		// combination of ADMXRC_DMAFLAG_XXX
									DWORD*					timeout,	// in microseconds
									PHANDLE					event);		// per-thread event handle

ADMXRC_DLLFUNC ADMXRC_STATUS __cdecl
ADMXRC_DoDMAImmediate(				ADMXRC_HANDLE			card,
									void*					buffer,		// target for DMA
									unsigned long			length,		// size of buffer
									DWORD					local,		// local bus address for DMA
									DWORD					direction,	// type is ADMXRC_DMA_DIRECTION
									DWORD					channel,	// type is ADMXRC_DMA_CHANNEL
									DWORD					mode,		// from ADMXRC_BuildDMAModeWord()
									DWORD					flags,		// combination of ADMXRC_DMAFLAG_XXX
									DWORD*					timeout,	// timeout in microseconds
									PHANDLE					event);		// per-thread event handle

/*
** Utility functions
*/

ADMXRC_DLLFUNC DWORD __cdecl
ADMXRC_BuildDMAModeWord(			DWORD					buswidth,	// type is ADMXRC_DMA_WIDTH
									DWORD					waitstates,
									DWORD					misc);		// combination of ADMXRC_DMAMODE_XXX

ADMXRC_DLLFUNC ADMXRC_STATUS __cdecl
ADMXRC_StatusToString(				ADMXRC_STATUS			status,
									char*					string,
									unsigned long			length);

ADMXRC_DLLFUNC const char* __cdecl
ADMXRC_GetStatusString(				ADMXRC_STATUS Status);

ADMXRC_DLLFUNC void __cdecl
ADMXRC_ReverseBytes(				ADMXRC_IMAGE			image,
									ULONG					offset,
									ULONG					length);

ADMXRC_DLLFUNC ADMXRC_STATUS __cdecl
ADMXRC_FindImageOffset(				ADMXRC_FPGA_TYPE		FpgaType,
									ADMXRC_IMAGE			image,
									ULONG					size,
									ULONG*					offs);

ADMXRC_DLLFUNC ADMXRC_STATUS __cdecl
ADMXRC_LoadFpgaFile(				UCHAR*					name,
									ADMXRC_IMAGE*			image,
									ULONG*					BytesRead);

ADMXRC_DLLFUNC ADMXRC_STATUS __cdecl
ADMXRC_UnloadFpgaFile(				ADMXRC_IMAGE			image);

#ifdef __cplusplus
} 
#endif

#endif /* ADMXRC_H */
