/*
** status.h - status codes returned by ADM-XRC API calls
**
** (C) Copyright 2000 Alpha Data Parallel Systems Ltd.
*/

#ifndef _ADATA_ADMXRC_STATUS_H
#define _ADATA_ADMXRC_STATUS_H

/*
** Base value for error status codes
*/
#define ADMXRC_STATUS_ERROR_CODE  (0x1000)

/*
** Status and error codes
*/
typedef enum 
	{
	ADMXRC_SUCCESS              = 0,

	// An error in the API logic occurred
	ADMXRC_INTERNAL_ERROR		= ADMXRC_STATUS_ERROR_CODE,

	// Couldn't allocate memory required to complete operation
	ADMXRC_NO_MEMORY,

	// Failed to open the card with specified CardID
	ADMXRC_CARD_NOT_FOUND,

	// Failed to open bitstream file
	ADMXRC_FILE_NOT_FOUND,

	// The bitstream file appears to be corrupt
	ADMXRC_INVALID_FILE,

	// The bitstream file does not match the FPGA on the card
	ADMXRC_FPGA_MISMATCH,

	// The handle to the card passed was invalid
	ADMXRC_INVALID_HANDLE,

	// The operation was not completed within the timeout period
	ADMXRC_TIMEOUT,

	// Card could not be opened because it was already open
	ADMXRC_CARD_BUSY,

	// An invalid parameter was supplied to the call
	ADMXRC_INVALID_PARAMETER,

	// The card was closed before the operation was completed
	ADMXRC_CLOSED,

	// A hardware error occurred on the card
	ADMXRC_CARD_ERROR,

	// An operation was requested which is not supported or implemented
	ADMXRC_NOT_SUPPORTED,

	// The requested device or resource was in use
	ADMXRC_DEVICE_BUSY,

	// The DMA descriptor passed was invalid
	ADMXRC_INVALID_DMADESC,

	// No free DMA descriptors left
	ADMXRC_NO_DMADESC,

	// The operation failed
	ADMXRC_FAILED,

	// The operation is still in progress
	ADMXRC_PENDING,

	// The operation failed for reasons unknown
	ADMXRC_UNKNOWN_ERROR,

	// A null pointer was supplied in the call
	ADMXRC_NULL_POINTER,

	// The operation was cancelled because requesting thread terminated
	ADMXRC_CANCELLED,

	// The driver revision level is too low
	ADMXRC_BAD_DRIVER

	} ADMXRC_STATUS;

#define ADMXRC_IS_ERROR(Status) ((Status) >= ADMXRC_STATUS_ERROR_CODE)

#endif

